shader_type spatial;
render_mode diffuse_burley, specular_schlick_ggx;

uniform bool useplanealbedo = true;
uniform bool useplanenormal = true;

uniform vec4 planealbedoColor : hint_color;
uniform sampler2D plane_albedo: hint_albedo;
// Ambient occlusion, roughness, metalness
uniform sampler2D plane_normal : hint_normal;
uniform sampler2D snow_albedo: hint_albedo;
uniform sampler2D snow_normal : hint_normal;
uniform float blend_smoothness : hint_range(0.1, 1.0, 0.01) = 0.2;
uniform float snow_range : hint_range(0.1, 1.0, 0.01) = 0.0;
uniform vec3 world_direction = vec3(0.0, 1.0, 0.0);

uniform vec2 planeUV_scale;
uniform vec2 planeUV_offset;

uniform vec2 snowUV_scale;
uniform vec2 snowUV_offset;

varying vec2 snowUV;
varying vec2 planeUV;

void vertex() {
	snowUV = UV * snowUV_scale.xy + snowUV_offset.xy;
	planeUV = UV * planeUV_scale.xy + planeUV_offset.xy;
}

void fragment(){
	// sample all the textures
	vec4 plane_albedotex = texture(plane_albedo, planeUV)*planealbedoColor;
	vec4 snow_albedotex = texture(snow_albedo, snowUV);
	vec3 plane_normalmap = texture(plane_normal, planeUV).rgb;
	vec3 snow_normalmap = texture(snow_normal, snowUV).rgb;
	float mix_factor = 0.0;
	float addends = 0.0;

	vec3 snow_world_normalmap = vec3(0,0,0);
	
	// textures are from 0 to 1, but we want the normals to be able to go negative.
	vec3 unpacked_normal = snow_normalmap * 2.0 - vec3(1.0);
	unpacked_normal.z = sqrt(max(0.0, 1.0 - dot(unpacked_normal.xy, unpacked_normal.xy)));
	// Binormal and tangent vectors are what allows us to transform the
	// 2D normalmaps into something that can interact with 3D space
	vec3 world_tangent = (CAMERA_MATRIX * vec4(TANGENT, 0.0)).xyz;
	vec3 world_binormal = (CAMERA_MATRIX * vec4(BINORMAL, 0.0)).xyz;
	vec3 world_normal = (CAMERA_MATRIX * vec4(NORMAL, 0.0)).xyz;
	// Normal mixing taken directly from Godot's spatial shader (drivers/gles3/shaders/spatial.glsl)
	snow_world_normalmap = normalize(unpacked_normal.y * world_tangent + unpacked_normal.x * world_binormal + world_normal * unpacked_normal.z);
		
	// the more the world-space normal is aligned to the given vector,
	// the more we want to show the second texture.
	mix_factor += clamp(
		dot(snow_world_normalmap, world_direction)
		,0.0,1.0);
	addends += 1.0;

	// This part makes the mix factor more crisp than a bland mix between zero and one.
	// threshold allows us to cut away more of the second texture, while smoothness defines
	// the crispiness of the edges of the blending. The smoother the blending, the more diluted
	// and blended the textures will look like.
	// It would be interesting to use the displacement map of the second texture as a small
	// boost to the mix_factor, so that blending can follow the shapes in the second texture.
	// Here are the textures: https://cc0textures.com/view?id=Moss003
	// Have fun!
	mix_factor = smoothstep(0.0, blend_smoothness, mix_factor - snow_range);

	if(useplanenormal){
		NORMALMAP = mix(plane_normalmap, snow_normalmap, mix_factor);
	}else{
		NORMALMAP = mix(NORMAL, snow_normalmap, mix_factor);
	}
			//NORMALMAP = mix(plane_normalmap, snow_normalmap, mix_factor);

	if(useplanealbedo){
		ALBEDO = clamp(mix(plane_albedotex, snow_albedotex, mix_factor).rgb, vec3(0.0), vec3(1.0));
	}else{
		ALBEDO = clamp(mix(planealbedoColor, snow_albedotex, mix_factor).rgb, vec3(0.0), vec3(1.0));
	}

	//ALBEDO = plane_albedotex.rgb;
}